@@ -23,7 +23,11 @@
 
 #define PY_SSIZE_T_CLEAN
 #include <Python.h>
+#if defined(__APPLE__)
+#include <sys/xattr.h>
+#elif defined(__linux__)
 #include <attr/xattr.h>
+#endif
 #include <stdio.h>
 
 /* Compatibility with python 2.4 regarding python size type (PEP 353) */
@@ -36,8 +40,10 @@ typedef int Py_ssize_t;
 #if PY_MAJOR_VERSION >= 3
 #define IS_PY3K
 #define BYTES_CHAR "y"
+#define BYTES_TUPLE "yy#"
 #else
 #define BYTES_CHAR "s"
+#define BYTES_TUPLE "ss#"
 #define PyBytes_Check PyString_Check
 #define PyBytes_AS_STRING PyString_AS_STRING
 #define PyBytes_FromStringAndSize PyString_FromStringAndSize
@@ -94,9 +100,14 @@ typedef int Py_ssize_t;
     "   string (byte string under Python 3)."
 
 
-/* the estimated (startup) attribute buffer size in
-   multi-operations */
-#define ESTIMATE_ATTR_SIZE 256
+/* The initial I/O buffer size for list and get operations; if the
+ * actual values will be smaller than this, we save a syscall out of
+ * two and allocate more memory upfront than needed, otherwise we
+ * incur three syscalls (get with ENORANGE, get with 0 to compute
+ * actual size, final get). The test suite is marginally faster (5%)
+ * with this, so it seems worth doing.
+*/
+#define ESTIMATE_ATTR_SIZE 1024
 
 typedef enum {T_FD, T_PATH, T_LINK} target_e;
 
@@ -163,6 +174,8 @@ static int convert_obj(PyObject *myobj, target_t *tgt, int nofollow) {
         tgt->fd = fd;
     } else {
         PyErr_SetString(PyExc_TypeError, "argument must be string or int");
+        tgt->type = T_PATH;
+        tgt->name = NULL;
         return -1;
     }
     return 0;
@@ -196,42 +209,203 @@ static int merge_ns(const char *ns, const char *name,
     return 0;
 }
 
-static ssize_t _list_obj(target_t *tgt, char *list, size_t size) {
+#if defined(__APPLE__)
+static inline ssize_t _listxattr(const char *path, char *namebuf, size_t size) {
+    return listxattr(path, namebuf, size, 0);
+}
+static inline ssize_t _llistxattr(const char *path, char *namebuf, size_t size) {
+    return listxattr(path, namebuf, size, XATTR_NOFOLLOW);
+}
+static inline ssize_t _flistxattr(int fd, char *namebuf, size_t size) {
+    return flistxattr(fd, namebuf, size, 0);
+}
+
+static inline ssize_t _getxattr (const char *path, const char *name, void *value, size_t size) {
+    return getxattr(path, name, value, size, 0, 0);
+}
+static inline ssize_t _lgetxattr (const char *path, const char *name, void *value, size_t size) {
+    return getxattr(path, name, value, size, 0, XATTR_NOFOLLOW);
+}
+static inline ssize_t _fgetxattr (int filedes, const char *name, void *value, size_t size) {
+    return fgetxattr(filedes, name, value, size, 0, 0);
+}
+
+// [fl]setxattr: Both OS X and Linux define XATTR_CREATE and XATTR_REPLACE for the last option.
+static inline int _setxattr(const char *path, const char *name, const void *value, size_t size, int flags) {
+    return setxattr(path, name, value, size, 0, flags);
+}
+static inline int _lsetxattr(const char *path, const char *name, const void *value, size_t size, int flags) {
+    return setxattr(path, name, value, size, 0, flags & XATTR_NOFOLLOW);
+}
+static inline int _fsetxattr(int filedes, const char *name, const void *value, size_t size, int flags) {
+    return fsetxattr(filedes, name, value, size, 0, flags);
+}
+
+static inline int _removexattr(const char *path, const char *name) {
+    return removexattr(path, name, 0);
+}
+static inline int _lremovexattr(const char *path, const char *name) {
+    return removexattr(path, name, XATTR_NOFOLLOW);
+}
+static inline int _fremovexattr(int filedes, const char *name) {
+    return fremovexattr(filedes, name, 0);
+}
+
+#elif defined(__linux__)
+#define _listxattr(path, list, size) listxattr(path, list, size)
+#define _llistxattr(path, list, size) llistxattr(path, list, size)
+#define _flistxattr(fd, list, size) flistxattr(fd, list, size)
+
+#define _getxattr(path, name, value, size)  getxattr(path, name, value, size)
+#define _lgetxattr(path, name, value, size) lgetxattr(path, name, value, size)
+#define _fgetxattr(fd, name, value, size)   fgetxattr(fd, name, value, size)
+
+#define _setxattr(path, name, value, size, flags)   setxattr(path, name, value, size, flags)
+#define _lsetxattr(path, name, value, size, flags)  lsetxattr(path, name, value, size, flags)
+#define _fsetxattr(fd, name, value, size, flags)    fsetxattr(fd, name, value, size, flags)
+
+#define _removexattr(path, name)    removexattr(path, name)
+#define _lremovexattr(path, name)   lremovexattr(path, name)
+#define _fremovexattr(fd, name)     fremovexattr(fd, name)
+
+#endif
+
+typedef ssize_t (*buf_getter)(target_t *tgt, const char *name,
+                              void *output, size_t size);
+
+static ssize_t _list_obj(target_t *tgt, const char *unused, void *list,
+                         size_t size) {
     if(tgt->type == T_FD)
-        return flistxattr(tgt->fd, list, size);
+        return _flistxattr(tgt->fd, list, size);
     else if (tgt->type == T_LINK)
-        return llistxattr(tgt->name, list, size);
+        return _llistxattr(tgt->name, list, size);
     else
-        return listxattr(tgt->name, list, size);
+        return _listxattr(tgt->name, list, size);
 }
 
 static ssize_t _get_obj(target_t *tgt, const char *name, void *value,
                         size_t size) {
     if(tgt->type == T_FD)
-        return fgetxattr(tgt->fd, name, value, size);
+        return _fgetxattr(tgt->fd, name, value, size);
     else if (tgt->type == T_LINK)
-        return lgetxattr(tgt->name, name, value, size);
+        return _lgetxattr(tgt->name, name, value, size);
     else
-        return getxattr(tgt->name, name, value, size);
+        return _getxattr(tgt->name, name, value, size);
 }
 
 static int _set_obj(target_t *tgt, const char *name,
                     const void *value, size_t size, int flags) {
     if(tgt->type == T_FD)
-        return fsetxattr(tgt->fd, name, value, size, flags);
+        return _fsetxattr(tgt->fd, name, value, size, flags);
     else if (tgt->type == T_LINK)
-        return lsetxattr(tgt->name, name, value, size, flags);
+        return _lsetxattr(tgt->name, name, value, size, flags);
     else
-        return setxattr(tgt->name, name, value, size, flags);
+        return _setxattr(tgt->name, name, value, size, flags);
 }
 
 static int _remove_obj(target_t *tgt, const char *name) {
     if(tgt->type == T_FD)
-        return fremovexattr(tgt->fd, name);
+        return _fremovexattr(tgt->fd, name);
     else if (tgt->type == T_LINK)
-        return lremovexattr(tgt->name, name);
+        return _lremovexattr(tgt->name, name);
     else
-        return removexattr(tgt->name, name);
+        return _removexattr(tgt->name, name);
+}
+
+/* Perform a get/list operation with appropriate buffer size,
+ * determined dynamically.
+ *
+ * Arguments:
+ * - getter: the function that actually does the I/O.
+ * - tgt, name: passed to the getter.
+ * - buffer: pointer to either an already allocated memory area (in
+ *   which case size contains its current size), or NULL to
+ *   allocate. In all cases (success or failure), the caller should
+ *   deallocate the buffer, using PyMem_Free(). Note that if size is
+ *   zero but buffer already points to allocate memory, it will be
+ *   ignored/leaked.
+ * - size: either size of current buffer (if non-NULL), or size for
+ *   initial allocation (if non-zero), or a zero value which means
+ *   auto-allocate buffer with automatically queried size. Value will
+ *   be updated upon return with the current buffer size.
+ * - io_errno: if non-NULL, the actual errno will be recorded here; if
+ *   zero, the call was successful and the output/size/nval are valid.
+ *
+ * Return value: if positive or zero, buffer will contain the read
+ * value. Otherwise, io_errno will contain the I/O errno, or zero
+ * to signify a Python-level error. In all cases, the Python-level
+ * error is set to the appropriate value.
+ */
+static ssize_t _generic_get(buf_getter getter, target_t *tgt,
+                            const char *name,
+                            char **buffer,
+                            size_t *size,
+                            int *io_errno)
+    CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION;
+
+static ssize_t _generic_get(buf_getter getter, target_t *tgt,
+                            const char *name,
+                            char **buffer,
+                            size_t *size,
+                            int *io_errno) {
+  ssize_t res;
+  /* Clear errno for now, will only set it when it fails in I/O. */
+  if (io_errno != NULL) {
+    *io_errno = 0;
+  }
+
+#define EXIT_IOERROR()                 \
+  {                                    \
+    if (io_errno != NULL) {            \
+        *io_errno = errno;             \
+    }                                  \
+    PyErr_SetFromErrno(PyExc_IOError); \
+    return -1;                         \
+  }
+
+  /* Initialize the buffer, if needed. */
+  if (*size == 0 || *buffer == NULL) {
+    if (*size == 0) {
+      ssize_t nalloc;
+      if ((nalloc = getter(tgt, name, NULL, 0)) == -1) {
+        EXIT_IOERROR();
+      }
+      if (nalloc == 0) {
+        /* Empty, so no need to retrieve it. */
+        return 0;
+      }
+      *size = nalloc;
+    }
+    if((*buffer = PyMem_Malloc(*size)) == NULL) {
+      PyErr_NoMemory();
+      return -1;
+    }
+  }
+  // Try to get the value, while increasing the buffer if too small.
+  while((res = getter(tgt, name, *buffer, *size)) == -1) {
+    if(errno == ERANGE) {
+      ssize_t realloc_size_s = getter(tgt, name, NULL, 0);
+      /* ERANGE + proper size _should_ not fail, but... */
+      if(realloc_size_s == -1) {
+        EXIT_IOERROR();
+      }
+      size_t realloc_size = (size_t) realloc_size_s;
+      char *tmp_buf;
+      if((tmp_buf = PyMem_Realloc(*buffer, realloc_size)) == NULL) {
+        PyErr_NoMemory();
+        return -1;
+      }
+      *buffer = tmp_buf;
+      *size = realloc_size;
+      continue;
+    } else {
+      /* else we're dealing with a different error, which we
+         don't know how to handle nicely, so we return */
+      EXIT_IOERROR();
+    }
+  }
+  return res;
+#undef EXIT_IOERROR
 }
 
 /*
@@ -277,9 +451,9 @@ pygetxattr(PyObject *self, PyObject *args)
     target_t tgt;
     int nofollow = 0;
     char *attrname = NULL;
-    char *buf;
-    ssize_t nalloc_s, nret;
-    size_t nalloc;
+    char *buf = NULL;
+    ssize_t nret;
+    size_t nalloc = ESTIMATE_ATTR_SIZE;
     PyObject *res;
 
     /* Parse the arguments */
@@ -290,33 +464,17 @@ pygetxattr(PyObject *self, PyObject *args)
         goto free_arg;
     }
 
-    /* Find out the needed size of the buffer */
-    if((nalloc_s = _get_obj(&tgt, attrname, NULL, 0)) == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_tgt;
+    nret = _generic_get(_get_obj, &tgt, attrname, &buf, &nalloc, NULL);
+    if (nret == -1) {
+      res = NULL;
+      goto free_buf;
     }
-
-    nalloc = (size_t) nalloc_s;
-
-    /* Try to allocate the memory, using Python's allocator */
-    if((buf = PyMem_Malloc(nalloc)) == NULL) {
-        res = PyErr_NoMemory();
-        goto free_tgt;
-    }
-
-    /* Now retrieve the attribute value */
-    if((nret = _get_obj(&tgt, attrname, buf, nalloc)) == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_buf;
-    }
-
     /* Create the string which will hold the result */
     res = PyBytes_FromStringAndSize(buf, nret);
 
  free_buf:
     /* Free the buffer, now it is no longer needed */
     PyMem_Free(buf);
- free_tgt:
     free_tgt(&tgt);
  free_arg:
     PyMem_Free(attrname);
@@ -356,45 +514,29 @@ xattr_get(PyObject *self, PyObject *args, PyObject *keywds)
     int nofollow = 0;
     char *attrname = NULL, *namebuf;
     const char *fullname;
-    char *buf;
+    char *buf = NULL;
     const char *ns = NULL;
-    ssize_t nalloc_s, nret;
-    size_t nalloc;
-    PyObject *res;
+    ssize_t nret;
+    size_t nalloc = ESTIMATE_ATTR_SIZE;
+    PyObject *res = NULL;
     static char *kwlist[] = {"item", "name", "nofollow", "namespace", NULL};
 
     /* Parse the arguments */
     if (!PyArg_ParseTupleAndKeywords(args, keywds, "Oet|i" BYTES_CHAR, kwlist,
                                      &myarg, NULL, &attrname, &nofollow, &ns))
         return NULL;
+    res = NULL;
     if(convert_obj(myarg, &tgt, nofollow) < 0) {
-        res = NULL;
         goto free_arg;
     }
 
     if(merge_ns(ns, attrname, &fullname, &namebuf) < 0) {
-        res = NULL;
         goto free_tgt;
     }
 
-    /* Find out the needed size of the buffer */
-    if((nalloc_s = _get_obj(&tgt, fullname, NULL, 0)) == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_name_buf;
-    }
-
-    nalloc = (size_t) nalloc_s;
-
-    /* Try to allocate the memory, using Python's allocator */
-    if((buf = PyMem_Malloc(nalloc)) == NULL) {
-        res = PyErr_NoMemory();
-        goto free_name_buf;
-    }
-
-    /* Now retrieve the attribute value */
-    if((nret = _get_obj(&tgt, fullname, buf, nalloc)) == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_buf;
+    nret = _generic_get(_get_obj, &tgt, fullname, &buf, &nalloc, NULL);
+    if(nret == -1) {
+      goto free_buf;
     }
 
     /* Create the string which will hold the result */
@@ -403,7 +545,6 @@ xattr_get(PyObject *self, PyObject *args, PyObject *keywds)
     /* Free the buffers, they are no longer needed */
  free_buf:
     PyMem_Free(buf);
- free_name_buf:
     PyMem_Free(namebuf);
  free_tgt:
     free_tgt(&tgt);
@@ -458,13 +599,14 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
     PyObject *myarg, *res;
     int nofollow=0;
     const char *ns = NULL;
-    char *buf_list, *buf_val, *buf_val_tmp;
+    char *buf_list = NULL, *buf_val = NULL;
     const char *s;
-    ssize_t nalloc_s, nlist, nval_s;
-    size_t nalloc, nval;
+    size_t nalloc = ESTIMATE_ATTR_SIZE;
+    ssize_t nlist, nval;
     PyObject *mylist;
     target_t tgt;
     static char *kwlist[] = {"item", "nofollow", "namespace", NULL};
+    int io_errno;
 
     /* Parse the arguments */
     if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|i" BYTES_CHAR, kwlist,
@@ -473,114 +615,48 @@ get_all(PyObject *self, PyObject *args, PyObject *keywds)
     if(convert_obj(myarg, &tgt, nofollow) < 0)
         return NULL;
 
+    res = NULL;
     /* Compute first the list of attributes */
-
-    /* Find out the needed size of the buffer for the attribute list */
-    nalloc_s = _list_obj(&tgt, NULL, 0);
-
-    if(nalloc_s == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_tgt;
+    nlist = _generic_get(_list_obj, &tgt, NULL, &buf_list,
+                         &nalloc, &io_errno);
+    if (nlist == -1) {
+      /* We can't handle any errors, and the Python error is already
+         set, just bail out. */
+      goto free_tgt;
     }
 
-    if(nalloc_s == 0) {
-        res = PyList_New(0);
-        goto free_tgt;
-    }
-
-    nalloc = (size_t) nalloc_s;
-
-    /* Try to allocate the memory, using Python's allocator */
-    if((buf_list = PyMem_Malloc(nalloc)) == NULL) {
-        res = PyErr_NoMemory();
-        goto free_tgt;
-    }
-
-    /* Now retrieve the list of attributes */
-    nlist = _list_obj(&tgt, buf_list, nalloc);
-
-    if(nlist == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_buf_list;
-    }
-
-    /* Create the list which will hold the result */
+    /* Create the list which will hold the result. */
     mylist = PyList_New(0);
     if(mylist == NULL) {
-      res = NULL;
       goto free_buf_list;
     }
 
     nalloc = ESTIMATE_ATTR_SIZE;
-    if((buf_val = PyMem_Malloc(nalloc)) == NULL) {
-        Py_DECREF(mylist);
-        res = PyErr_NoMemory();
-        goto free_buf_list;
-    }
-
     /* Create and insert the attributes as strings in the list */
     for(s = buf_list; s - buf_list < nlist; s += strlen(s) + 1) {
         PyObject *my_tuple;
-        int missing;
         const char *name;
 
-        if((name=matches_ns(ns, s))==NULL)
+        if((name = matches_ns(ns, s)) == NULL)
             continue;
         /* Now retrieve the attribute value */
-        missing = 0;
-        while(1) {
-            nval_s = _get_obj(&tgt, s, buf_val, nalloc);
-
-            if(nval_s == -1) {
-                if(errno == ERANGE) {
-                    ssize_t realloc_size_s = _get_obj(&tgt, s, NULL, 0);
-                    /* ERANGE + proper size should not fail, but it
-                       still can, so let's check first */
-                    if(realloc_size_s == -1) {
-                      res = PyErr_SetFromErrno(PyExc_IOError);
-                      Py_DECREF(mylist);
-                      goto free_buf_val;
-                    }
-                    size_t realloc_size = (size_t) realloc_size_s;
-                    if((buf_val_tmp = PyMem_Realloc(buf_val, realloc_size))
-                       == NULL) {
-                        res = PyErr_NoMemory();
-                        Py_DECREF(mylist);
-                        goto free_buf_val;
-                    }
-                    buf_val = buf_val_tmp;
-                    nalloc = realloc_size;
-                    continue;
-                } else if(
+        nval = _generic_get(_get_obj, &tgt, s, &buf_val, &nalloc, &io_errno);
+        if (nval == -1) {
+          if (
 #ifdef ENODATA
-                          errno == ENODATA ||
+              io_errno == ENODATA ||
 #endif
-                          errno == ENOATTR) {
-                    /* this attribute has gone away since we queried
-                       the attribute list */
-                    missing = 1;
-                    break;
-                }
-                /* else we're dealing with a different error, which we
-                   don't know how to handle nicely, so we abort */
-                Py_DECREF(mylist);
-                res = PyErr_SetFromErrno(PyExc_IOError);
-                goto free_buf_val;
-            } else {
-              nval = (size_t) nval_s;
-            }
-            break;
-        }
-        if(missing)
+              io_errno == ENOATTR) {
+            PyErr_Clear();
             continue;
-#ifdef IS_PY3K
-        my_tuple = Py_BuildValue("yy#", name, buf_val, nval);
-#else
-        my_tuple = Py_BuildValue("ss#", name, buf_val, nval);
-#endif
+          } else {
+            Py_DECREF(mylist);
+            goto free_buf_val;
+          }
+        }
+        my_tuple = Py_BuildValue(BYTES_TUPLE, name, buf_val, nval);
         if (my_tuple == NULL) {
           Py_DECREF(mylist);
-          res = NULL;
           goto free_buf_val;
         }
         PyList_Append(mylist, my_tuple);
@@ -902,10 +978,10 @@ static char __pylistxattr_doc__[] =
 static PyObject *
 pylistxattr(PyObject *self, PyObject *args)
 {
-    char *buf;
-    int nofollow=0;
-    ssize_t nalloc_s, nret;
-    size_t nalloc;
+    char *buf = NULL;
+    int nofollow = 0;
+    ssize_t nret;
+    size_t nalloc = ESTIMATE_ATTR_SIZE;
     PyObject *myarg;
     PyObject *mylist;
     Py_ssize_t nattrs;
@@ -918,29 +994,10 @@ pylistxattr(PyObject *self, PyObject *args)
     if(convert_obj(myarg, &tgt, nofollow) < 0)
         return NULL;
 
-    /* Find out the needed size of the buffer */
-    if((nalloc_s = _list_obj(&tgt, NULL, 0)) == -1) {
-        mylist = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_tgt;
-    }
-
-    if(nalloc_s == 0) {
-        mylist = PyList_New(0);
-        goto free_tgt;
-    }
-
-    nalloc = (size_t) nalloc_s;
-
-    /* Try to allocate the memory, using Python's allocator */
-    if((buf = PyMem_Malloc(nalloc)) == NULL) {
-        mylist = PyErr_NoMemory();
-        goto free_tgt;
-    }
-
-    /* Now retrieve the list of attributes */
-    if((nret = _list_obj(&tgt, buf, nalloc)) == -1) {
-        mylist = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_buf;
+    nret = _generic_get(_list_obj, &tgt, NULL, &buf, &nalloc, NULL);
+    if (nret == -1) {
+      mylist = NULL;
+      goto free_buf;
     }
 
     /* Compute the number of attributes in the list */
@@ -950,8 +1007,9 @@ pylistxattr(PyObject *self, PyObject *args)
 
     /* Create the list which will hold the result */
     mylist = PyList_New(nattrs);
-    if(mylist == NULL)
+    if(mylist == NULL) {
       goto free_buf;
+    }
 
     /* Create and insert the attributes as strings in the list */
     for(s = buf, nattrs = 0; s - buf < nret; s += strlen(s) + 1) {
@@ -968,8 +1026,6 @@ pylistxattr(PyObject *self, PyObject *args)
  free_buf:
     /* Free the buffer, now it is no longer needed */
     PyMem_Free(buf);
-
- free_tgt:
     free_tgt(&tgt);
 
     /* Return the result */
@@ -1005,10 +1061,10 @@ static char __list_doc__[] =
 static PyObject *
 xattr_list(PyObject *self, PyObject *args, PyObject *keywds)
 {
-    char *buf;
+    char *buf = NULL;
     int nofollow = 0;
-    ssize_t nalloc_s, nret;
-    size_t nalloc;
+    ssize_t nret;
+    size_t nalloc = ESTIMATE_ATTR_SIZE;
     PyObject *myarg;
     PyObject *res;
     const char *ns = NULL;
@@ -1021,34 +1077,13 @@ xattr_list(PyObject *self, PyObject *args, PyObject *keywds)
     if (!PyArg_ParseTupleAndKeywords(args, keywds, "O|i" BYTES_CHAR, kwlist,
                                      &myarg, &nofollow, &ns))
         return NULL;
+    res = NULL;
     if(convert_obj(myarg, &tgt, nofollow) < 0) {
-        res = NULL;
         goto free_arg;
     }
-
-    /* Find out the needed size of the buffer */
-    if((nalloc_s = _list_obj(&tgt, NULL, 0)) == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_tgt;
-    }
-
-    if(nalloc_s == 0) {
-        res = PyList_New(0);
-        goto free_tgt;
-    }
-
-    nalloc = (size_t) nalloc_s;
-
-    /* Try to allocate the memory, using Python's allocator */
-    if((buf = PyMem_Malloc(nalloc)) == NULL) {
-        res = PyErr_NoMemory();
-        goto free_tgt;
-    }
-
-    /* Now retrieve the list of attributes */
-    if((nret = _list_obj(&tgt, buf, nalloc)) == -1) {
-        res = PyErr_SetFromErrno(PyExc_IOError);
-        goto free_buf;
+    nret = _generic_get(_list_obj, &tgt, NULL, &buf, &nalloc, NULL);
+    if (nret == -1) {
+      goto free_tgt;
     }
 
     /* Compute the number of attributes in the list */
@@ -1056,15 +1091,16 @@ xattr_list(PyObject *self, PyObject *args, PyObject *keywds)
         if(matches_ns(ns, s) != NULL)
             nattrs++;
     }
+
     /* Create the list which will hold the result */
-    res = PyList_New(nattrs);
-    if(res == NULL)
+    if((res = PyList_New(nattrs)) == NULL) {
         goto free_buf;
+    }
 
     /* Create and insert the attributes as strings in the list */
     for(s = buf, nattrs = 0; s - buf < nret; s += strlen(s) + 1) {
         const char *name = matches_ns(ns, s);
-        if(name!=NULL) {
+        if(name != NULL) {
             PyObject *item = PyBytes_FromString(name);
             if(item == NULL) {
                 Py_DECREF(res);
